package cn.iocoder.yudao.module.member.service.user;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.member.api.user.dto.UserSelectReqVO;
import cn.iocoder.yudao.module.member.controller.admin.user.vo.UserBackVO;
import cn.iocoder.yudao.module.member.controller.admin.user.vo.UserQueryVO;
import cn.iocoder.yudao.module.member.controller.app.auth.vo.AppEmailUpdateReqVO;
import cn.iocoder.yudao.module.member.vo.member.MemberUpdateReqVO;
import cn.iocoder.yudao.module.member.controller.app.user.vo.AppUserUpdateMobileReqVO;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;

import javax.validation.Valid;
import java.io.InputStream;
import java.util.Collection;
import java.util.List;

/**
 * 会员用户 Service 接口
 *
 * @author 捷道源码
 */
public interface MemberUserService {

    /**
     * 通过手机查询用户
     *
     * @param mobile 手机
     * @return 用户对象
     */
    MemberUserDO getUserByMobile(String mobile);

    MemberUserDO checkPhoneUnique(String areaCode, String mobile);

    MemberUserDO getUserByMobile(String areaCode, String mobile);

    MemberUserDO getUserByEmail(String email);

    /**
     * 基于手机号创建用户。
     * 如果用户已经存在，则直接进行返回
     *
     * @param mobile     手机号
     * @param registerIp 注册 IP
     * @return 用户对象
     */
    MemberUserDO createUserIfAbsent(String mobile, String areaCode, String registerIp,Integer platform);

    /****
     * 基于手机号创建用户
     * @param mobile
     * @param registerIp
     * @param password
     * @param referralCode
     * @return
     */
    MemberUserDO createUser(String mobile, String registerIp, String password, String areaCode, String englishName, String referralCode, Integer registerPlatform);

    /**
     * 更新用户的最后登陆信息
     *
     * @param id      用户编号
     * @param loginIp 登陆 IP
     */
    void updateUserLogin(Long id, String loginIp);

    /**
     * 通过用户 ID 查询用户
     *
     * @param id 用户ID
     * @return 用户对象信息
     */
    MemberUserDO getUser(Long id);

    /**
     * 通过用户 ID 查询用户
     *
     * @param id 用户ID
     * @return 用户对象信息, 包括公司信息
     */
    MemberUserDO info(Long id);


    List<MemberUserDO> selectList(Collection<Long> ids);


    PageResult<MemberUserDO> selectList(UserSelectReqVO userSelectReqVO);

    /**
     * 修改用户昵称
     *
     * @param userId   用户id
     * @param nickname 用户新昵称
     */
    void updateUserNickname(Long userId, String nickname);

    /**
     * 修改用户头像
     *
     * @param userId      用户id
     * @param inputStream 头像文件
     * @return 头像url
     */
    String updateUserAvatar(Long userId, InputStream inputStream) throws Exception;

    /**
     * 修改手机
     *
     * @param userId 用户id
     * @param reqVO  请求实体
     */
    void updateUserMobile(Long userId, AppUserUpdateMobileReqVO reqVO);

    /**
     * 获得用户分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 用户分页
     */
    PageResult<MemberUserDO> getUserPage(UserQueryVO query, PageVO page);

    /**
     * 获得用户列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 用户列表
     */
    List<MemberUserDO> getUserList(UserQueryVO query);

    /**
     * 根据id更新用户
     *
     * @param memberUserDO 用户对象
     * @return 更新数据量
     */
    int updateById(MemberUserDO memberUserDO);


    /**
     * 删除 用户
     *
     * @param id 用户id
     */
    void deleteById(Long id);

    /**
     * 更新用户
     *
     * @param updateReqVO 更新信息
     */
    void updateUser(@Valid MemberUpdateReqVO updateReqVO);


    /**
     * 更新头像新接口
     *
     * @param avatar 更新信息
     */
    void updateNewUserAvatar(Long loginUserId, String avatar);

    /**
     * 绑定邮箱接口
     *
     * @param loginUserId
     * @param reqVO
     */
    void updateEmail(Long loginUserId, AppEmailUpdateReqVO reqVO);


    /**
     * 控货订单统计
     * TODO 分布式系统可能需要在分库分表时将此方法实现切换到订单模块,现在未避免循环依赖故将实现方法放在此处,也可用使用监听去实现
     *
     * @param userId 会员ID
     * @param phone  会员手机号
     * @param type   1 全部控货订单数  2 控货中的订单数
     * @return 控货订单数量
     */
    Integer appCargoControlCount(Long userId, String phone, Integer type);

    /**
     * 会员发出的订单统计
     * TODO 分布式系统可能需要在分库分表时将此方法实现切换到订单模块,现在未避免循环依赖故将实现方法放在此处,也可用使用监听去实现
     *
     * @param mobile              用户手机号
     * @param userId              用户ID
     * @param customerContactsIds 客户联系人ids
     * @return 会员发出的订单数量统计
     */
    Integer appIssuedOrderCount(String mobile, Long userId, Collection<Long> customerContactsIds);

    /**
     * 会员收到的订单统计
     * TODO 分布式系统可能需要在分库分表时将此方法实现切换到订单模块,现在未避免循环依赖故将实现方法放在此处,也可用使用监听去实现
     *
     * @param phone               会员电话
     * @param customerContactsIds 客户联系人ids
     * @return 会员收到的订单数量统计
     */
    Integer appReceivedOrderCount(String phone, Collection<Long> customerContactsIds);

    /**
     * 会员收到的订单统计
     * TODO 分布式系统可能需要在分库分表时将此方法实现切换到订单模块,现在未避免循环依赖故将实现方法放在此处,也可用使用监听去实现
     *
     * @param userId              用户ID
     * @param phone               会员电话
     * @param customerContactsIds 客户联系人ids
     * @return 会员收到的订单数量统计
     */
    Integer appAllOrderCount(Long userId, String phone, Collection<Long> customerContactsIds);

    List<UserBackVO> get7daysCount();

    PageResult<MemberUserDO> getWaitApproveUserPage(UserQueryVO query, PageVO page);

    void updateMemberUserDealStatus(Long customerId, Integer customerStatus);

    void memberCancellation(Long userId);

    List<MemberUserDO> getByCursor(Long startUserId, long limit);

    void historyCodeFlush();

    MemberUserDO getUserByCode(String referralCode);
}
